"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class CancelableRequest {
    constructor(cw, req) {
        this.cw = cw;
        this.req = req;
        this.thenCbs = [];
        this.catchCbs = [];
        this.cancelledCbs = [];
        this.gotCancel = false;
        this.req.subscribe((v) => {
            if (this.gotCancel) {
                this.cw.etcd.cfg.log.debug('cancelled');
                return;
            }
            this.thenCbs.forEach(tc => tc(v));
        }, (reason) => {
            this.catchCbs.forEach(rj => rj(reason));
        });
    }
    cancel() {
        this.gotCancel = true;
        this.thenCbs = [];
        this.catchCbs = [];
        const tmp = this.cancelledCbs;
        this.cancelledCbs = [];
        tmp.forEach(c => c());
    }
    cancelled(v) {
        this.cancelledCbs.push(v);
    }
    then(v) {
        this.thenCbs.push(v);
    }
    catch(v) {
        this.catchCbs.push(v);
    }
}
class ChangeWaiter {
    constructor(etcd, path, params = {}, options = {}) {
        this.etcd = etcd;
        this.path = path;
        this.rejects = [];
        this.fulfilled = [];
        this.options = options;
        this.params = params;
        this.runningRequests = [];
        this.getQueue = [];
    }
    unsubscribe() {
        this.rejects = [];
        this.fulfilled = [];
    }
    cancel() {
        this.runningRequests.forEach(r => r.cancel());
        this.unsubscribe();
        this.etcd.cfg.log.debug('cancelled', this.runningRequests.length);
    }
    findMaxModifiedIndex(n) {
        if (n.dir) {
            return n.nodes
                .map(c => this.findMaxModifiedIndex(c))
                .reduce((p, c) => Math.max(p, c), n.modifiedIndex);
        }
        else {
            return n.modifiedIndex;
        }
    }
    get(wait, waitIndex = null) {
        const params = Object.assign({}, this.params);
        params['wait'] = wait;
        if (waitIndex !== null) {
            params['waitIndex'] = waitIndex;
        }
        this.etcd.cfg.log.debug('get', waitIndex, this.runningRequests.length);
        const cancelable = new CancelableRequest(this, this.etcd.getRaw(this.path, params, this.options));
        this.runningRequests.push(cancelable);
        cancelable.cancelled(() => {
            this.runningRequests = this.runningRequests.filter(r => r !== cancelable);
        });
        cancelable.then((v) => {
            this.runningRequests = this.runningRequests.filter(r => r !== cancelable);
            if (v.action == 'error') {
                this.etcd.cfg.log.debug('retry', waitIndex);
                this.get(true, waitIndex);
            }
            else {
                this.current = v;
                this.get(true, this.findMaxModifiedIndex(v.node) + 1);
                // if cancelled the just issued runningRequest should
                // be cancelled also
                this.fulfilled.forEach(ff => ff(v)); // don't move
            }
        });
        cancelable.catch((reason) => {
            this.runningRequests = this.runningRequests.filter(r => r !== cancelable);
            this.rejects.forEach(rj => rj(reason));
        });
    }
    subscribe(onfulfilled, onrejected) {
        if (onfulfilled) {
            this.fulfilled.push(onfulfilled);
        }
        if (this.fulfilled.length == 1) {
            this.get(false);
        }
        else if (this.current) {
            onfulfilled(this.current);
        }
        if (onrejected) {
            this.rejects.push(onrejected);
        }
        return this;
    }
}
exports.ChangeWaiter = ChangeWaiter;
exports.default = ChangeWaiter;
//# sourceMappingURL=change-waiter.js.map