"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Uuid = require("uuid");
const os = require("os");
const path = require("path");
const rx = require("rxjs");
class WaitMaster {
    constructor(key, etc, ttl = 30000, saveTtl = 1000, startCb, stopCb) {
        this.reqStop = false;
        this.stopped = false;
        this.stopAction = null;
        this.master = false;
        this.startCb = null;
        this.stopCb = null;
        this.currentAliveTimeOut = null;
        this.currentWait = null;
        this.etc = etc;
        this.key = key;
        this.ttl = ttl;
        this.saveTtl = saveTtl;
        this.startCb = startCb || (() => { });
        this.stopCb = stopCb || (() => { });
    }
    static create(key, etc, ttl, saveTtl, start = null, stop = null) {
        const ret = new WaitMaster(key, etc, ttl, saveTtl, start, stop);
        return rx.Observable.create((obs) => {
            etc.mkdir(key).subscribe(() => {
                ret.me = `${os.hostname()}-${process.pid}-${Uuid.v4().toString()}`;
                setTimeout(ret.startWaitChange.bind(ret), 0);
                etc.addQueue(key, ret.me, ret.ttl + ret.saveTtl).subscribe(aq => {
                    ret.aliveQueue = `${key}/${path.basename(aq['node']['key'])}`;
                    ret.startAliveQueue();
                    obs.next(ret);
                    obs.complete();
                }, err => {
                    obs.error(err);
                });
            }, (err) => {
                if (err.statusCode != 403) {
                    etc.cfg.log.error(err); // Error:  105: Key already exists (/menox)
                    obs.error(`can not create directory:${key}`);
                }
            });
        });
    }
    renewTimeout() {
        if (this.reqStop) {
            return;
        }
        this.etc.update(this.aliveQueue, this.ttl + this.saveTtl).subscribe((_) => {
            this.startAliveQueue();
        }, (e) => {
            this.etc.cfg.log.error('update alive failed');
            this.stop();
        });
    }
    startAliveQueue() {
        this.currentAliveTimeOut = setTimeout(this.renewTimeout.bind(this), this.ttl);
    }
    stop() {
        if (this.reqStop) {
            this.etc.cfg.log.error('stop is running');
            return;
        }
        this.reqStop = true;
        if (this.currentAliveTimeOut) {
            clearTimeout(this.currentAliveTimeOut);
        }
        // console.log('ReqStop:', this.me, this.aliveQueue)
        this.etc.delete(this.aliveQueue).subscribe(() => {
            /* */
        }, (err) => {
            this.etc.cfg.log.error('stop delete failed', err);
        });
    }
    doStop(masterId) {
        this.master = false;
        this.stopped = true;
        if (this.stopAction) {
            this.stopAction();
            this.stopAction = null;
        }
        // console.log('Stop:', this.me, this.aliveQueue, masterId)
    }
    checkAmIMaster() {
        return rx.Observable.create((obs) => {
            this.etc.list(this.key, { recursive: true, sorted: true }).subscribe(master => {
                // console.log('master:', this.key, master)
                if (master.isErr()) {
                    obs.next(master);
                    obs.complete();
                    return;
                }
                let masterId = null;
                if (master.value.length != 0) {
                    masterId = `${this.key}/${path.basename(master.value[0]['key'])}`;
                }
                if (this.master && masterId != this.aliveQueue) {
                    this.doStop(masterId);
                    obs.next('got slave');
                    obs.complete();
                    return;
                }
                if (masterId == this.aliveQueue) {
                    if (this.master) {
                        obs.next('im the master');
                        obs.complete();
                        return;
                    }
                    this.master = true;
                    this.stopAction = this.stopCb;
                    this.startCb();
                    obs.next('got master');
                    obs.complete();
                    return;
                }
                if (this.reqStop && !this.master) {
                    this.stopped = true;
                    obs.next('slaved stopped');
                    obs.complete();
                    return;
                }
                obs.next('im a slave');
                obs.complete();
                return;
            }, (e) => {
                this.etc.cfg.log.error(e);
                obs.error(e);
                return;
            });
        });
    }
    startWaitChange() {
        let waitIndex = null;
        if (!this.stopped) {
            let params = { 'wait': true, 'recursive': true };
            if (waitIndex) {
                params['waitIndex'] = waitIndex;
            }
            // let master = this.master;
            // master && console.log('wait...', this.me, this.aliveQueue)
            this.currentWait = this.etc.getRaw(this.key, params, { timeout: 3600000 });
            this.currentWait.subscribe((ret) => {
                // master && console.log('await', this.me, this.aliveQueue)
                waitIndex = ret['node']['modifiedIndex'] + 1;
                this.checkAmIMaster().subscribe((_) => {
                    // console.log('startWaitChange:', _);
                    this.startWaitChange();
                }, (e) => {
                    this.etc.cfg.log.error(e);
                });
            }, (e) => {
                this.etc.cfg.log.error(e);
            });
        }
    }
}
exports.WaitMaster = WaitMaster;
//# sourceMappingURL=wait_master.js.map