"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Uuid = require("uuid");
const etcd = require("../lib/index");
/*
function param(arr: string[], uuid: string): string[] {
  return arr.concat(['--etcd-cluster-id', uuid, '--etcd-url', 'http://localhost:2379']);
}
*/
describe('etcd', function () {
    this.timeout(2000);
    before(() => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        console.log('etcd Cluster Booted...0');
        let etc = etcd.EtcdPromise.create(wc);
        console.log('etcd Cluster Booted...1');
        yield etc.connect();
        console.log('etcd Cluster Booted...2');
        return Promise.resolve('done');
    }));
    it('AsyncPromise Blocking', () => __awaiter(this, void 0, void 0, function* () {
        let value = 1;
        let out = new Promise((r, j) => __awaiter(this, void 0, void 0, function* () {
            yield new Promise((rr, jj) => { setTimeout(rr, 10); });
            ++value;
        }));
        out = null; // WTF Lint
        chai_1.assert.equal(value, 1);
        yield new Promise((rr, jj) => { setTimeout(rr, 15); });
        chai_1.assert.equal(value, 2);
        return Promise.resolve('done');
    }));
    it('selfStat one good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(1, ret.length);
        // console.log(ret[0])
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat two same good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(2, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        chai_1.assert.equal(ret[1].isOk(), true);
        chai_1.assert.equal(ret[1].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat same good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(4, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat connect', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true, 'isok');
        chai_1.assert.equal(ret.url, 'http://localhost:2379', 'url');
        return Promise.resolve('done');
    }));
    it('selfStat clusterState bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://169.254.99.94'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(1, ret.length);
        chai_1.assert.equal(ret[0].isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat connect bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://169.254.99.94',
            '--etcd-url', 'http://169.254.99.95'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat connect good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat connect two good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat connect two good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.55.99',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat one good one bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.99.94'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(2, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[1].isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat two good one bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.99.94',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(3, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[1].isOk(), false);
        chai_1.assert.equal(ret[2].isOk(), true);
        return Promise.resolve('done');
    }));
    it('mkdir', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.mkdir('meno');
        chai_1.assert.equal(ret.isErr(), false);
        ret = yield etc.mkdir('meno');
        chai_1.assert.equal(ret.isErr(), true);
        let lst = yield etc.list('meno');
        chai_1.assert.equal(lst.isErr(), false);
        chai_1.assert.equal(lst.value.length, 0);
        ret = yield etc.rmdir('meno');
        chai_1.assert.equal(ret.isOk(), true);
    }));
    it('rmdir', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.EtcdPromise.create(wc);
        yield etc.mkdir('meno');
        yield etc.mkdir('meno/geheim');
        yield etc.mkdir('meno/geheim/ganz');
        let ret = yield etc.rmdir('meno', { recursive: true });
        chai_1.assert.equal(ret.isOk(), true);
        yield etc.mkdir('meno');
        let lst = yield etc.list('meno');
        chai_1.assert.equal(lst.isErr(), false);
        chai_1.assert.equal(lst.value.length, 0);
        ret = yield etc.rmdir('meno', { recursive: true });
        chai_1.assert.equal(ret.isOk(), true);
    }));
    it('get/setRaw', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.EtcdPromise.create(wc);
        let ret = yield etc.getRaw('hammer/murk');
        chai_1.assert.equal(ret.isErr(), true);
        ret = yield etc.setRaw('hammer/murk', 'Hello World');
        chai_1.assert.equal(ret.isErr(), false);
        ret = yield etc.getRaw('hammer/murk');
        chai_1.assert.equal(ret.isOk(), true);
        // console.log(ret.node)
        chai_1.assert.equal(ret.node.value, 'Hello World');
    }));
    function leftPad(n, pad) {
        return ('' + (Math.pow(10, pad) + n)).substr(1);
    }
    function changeWriterThen(source, ccw, cnt, done) {
        let nested = 0;
        ccw.subscribe((er) => {
            nested++;
            let node = er.node;
            if (er.node.dir) {
                node = er.node.nodes.sort((a, b) => {
                    if (a.key < b.key) {
                        return -1;
                    }
                    if (a.key > b.key) {
                        return 1;
                    }
                    return 0;
                })[er.node.nodes.length - 1];
            }
            // console.log(cnt.cnt, node.key, node.value, nested);
            chai_1.assert.isTrue(node.key.endsWith(`wait-for-change/hallo-${leftPad(cnt.cnt, 5)}`), 'not match');
            chai_1.assert.equal(`vallo-${leftPad(cnt.cnt, 5)}`, node.value, 'funny value');
            cnt.cnt++;
            if (cnt.cnt && (cnt.cnt % 4) == 0) {
                done();
            }
            else {
                updateData(source, ccw, cnt, done, false);
            }
            --nested;
        }, () => {
            chai_1.assert.fail('should never called');
        });
    }
    function updateData(source, ccw, cnt, done, install) {
        setTimeout(() => {
            source.setRaw(`wait-for-change/hallo-${leftPad(cnt.cnt, 5)}`, `vallo-${leftPad(cnt.cnt, 5)}`).then(() => {
                if (install) {
                    changeWriterThen(source, ccw, cnt, done);
                }
            }).catch(() => {
                chai_1.assert.fail('should never called');
            });
        }, (cnt.cnt % 1000) * 90);
    }
    it('wait-for-change', (done) => {
        const uuid = Uuid.v4().toString();
        const wc = etcd.Config.start([
            '--etcd-cluster-id', uuid,
            '--etcd-req-timeout', '200'
        ]);
        const source = etcd.EtcdPromise.create(wc);
        source.mkdir('wait-for-change').then(() => {
            const ccw = source.createChangeWaiter('wait-for-change', { recursive: true });
            const cnt = { cnt: 0 };
            updateData(source, ccw, cnt, () => {
                ccw.cancel();
                cnt.cnt = 1000;
                setTimeout(() => updateData(source, ccw, cnt, () => {
                    ccw.cancel();
                    setTimeout(() => {
                        source.setRaw(`wait-for-change/cancelled`, `stopped`).then(() => {
                            done();
                        }).catch(() => {
                            chai_1.assert.fail('never called');
                        });
                    }, 100);
                }, true), 300);
            }, true);
        });
    });
});
//# sourceMappingURL=etcd-test.js.map