"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Uuid = require("uuid");
const etcd = require("../lib/index");
/*
function param(arr: string[], uuid: string): string[] {
  return arr.concat(['--etcd-cluster-id', uuid, '--etcd-url', 'http://localhost:2379']);
}
*/
function masterCount(wms) {
    let mc = 0;
    for (let i = 0; i < wms.length; ++i) {
        if (wms[i].master) {
            ++mc;
        }
    }
    return mc;
}
function stopCount(wms) {
    let sc = 0;
    for (let i = 0; i < wms.length; ++i) {
        if (wms[i].stopped) {
            ++sc;
        }
    }
    return sc;
}
function slaves(wms) {
    return wms.filter((wm) => !wm.master);
}
function master(wms) {
    return wms.find((wm) => wm.master);
}
function mastersSum(masters) {
    return masters.reduce((a, b) => a + b, 0);
}
describe('wait-master', function () {
    this.timeout(10000);
    it('elect-master', () => __awaiter(this, void 0, void 0, function* () {
        // this.timeout(10000)
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start([]);
        let etc = etcd.EtcdObserable.create(wc);
        let masters = [];
        let waitMasters = [];
        let totalWaiters = 10;
        for (let i = 0; i < totalWaiters; ++i) {
            masters[i] = 0;
            const wmc = yield etcd.WaitMaster.create(uuid, etc, 100, 100, ((id) => { return () => { ++masters[id]; }; })(i), ((id) => { return () => { --masters[id]; }; })(i)).toPromise();
            waitMasters.push(wmc);
        }
        yield new Promise((res, rej) => { setTimeout(res, 400); });
        chai_1.assert.equal(mastersSum(masters), 1, 'master count 1 A');
        chai_1.assert.equal(masterCount(waitMasters), 1, 'master count 1 B');
        chai_1.assert.equal(stopCount(waitMasters), 0, 'stop count 0 C');
        yield slaves(waitMasters)[0].stop();
        yield new Promise((res, rej) => { setTimeout(res, 150); });
        chai_1.assert.equal(mastersSum(masters), 1, 'master count 1 D');
        chai_1.assert.equal(masterCount(waitMasters), 1, 'master count 1 E');
        chai_1.assert.equal(stopCount(waitMasters), 1, 'stop count 1 F');
        yield master(waitMasters).stop();
        yield new Promise((res, rej) => { setTimeout(res, 200); });
        chai_1.assert.equal(mastersSum(masters), 1, 'master count 1 G');
        chai_1.assert.equal(masterCount(waitMasters), 1, 'master count 1 H');
        chai_1.assert.equal(stopCount(waitMasters), 2, 'stop count 1 I');
        let wm = yield etcd.WaitMaster.create(uuid, etc, 100, 100, () => chai_1.assert('will not get the master'), () => chai_1.assert('do not stop')).toPromise();
        yield new Promise((res, rej) => { setTimeout(res, 150); });
        yield wm.stop();
        chai_1.assert.equal(mastersSum(masters), 1, 'master count 1 J');
        chai_1.assert.equal(masterCount(waitMasters), 1, 'master count 1 K');
        chai_1.assert.equal(stopCount(waitMasters), 2, 'stop count 1 L');
        yield new Promise((res, rej) => { setTimeout(res, 150); });
        let cnt = 0;
        while (master(waitMasters)) {
            ++cnt;
            // console.log('>>>>stop current master', cnt)
            yield master(waitMasters).stop();
            yield new Promise((res, rej) => { setTimeout(res, 300); });
            // console.log('>>>>waited current master', master(waitMasters))
            if (master(waitMasters)) {
                chai_1.assert.equal(mastersSum(masters), 1, 'master count 1 M');
                chai_1.assert.equal(masterCount(waitMasters), 1, 'master count 1 N');
                chai_1.assert.equal(stopCount(waitMasters), 2 + cnt, `stop count cnt O ${cnt}`);
            }
            else {
                chai_1.assert.equal(mastersSum(masters), 0, 'master count 1 M');
                chai_1.assert.equal(masterCount(waitMasters), 0, 'master count 1 N');
                chai_1.assert.equal(stopCount(waitMasters), 2 + cnt, `stop count cnt O ${cnt}`);
            }
        }
        chai_1.assert.equal(totalWaiters, cnt + 2);
        // console.log('DONE')
        return Promise.resolve('done');
    }));
});
//# sourceMappingURL=wait_master-test.js.map